@@ -98,20 +98,6 @@ outline_window_expose (GtkWidget      *widget,
   return FALSE;
 }
 
-static char*
-utf8_strndup (const char *src,
-              int         n)
-{
-  const gchar *s = src;
-  while (n && *s)
-    {
-      s = g_utf8_next_char (s);
-      n--;
-    }
-
-  return g_strndup (src, s - src);
-}
-
 static GdkPixbuf*
 dimm_icon (GdkPixbuf *pixbuf)
 {
@@ -238,7 +224,7 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
               tmp = markup;
             }
             
-          te->title = utf8_strndup (tmp, max_chars_per_title);
+          te->title = meta_g_utf8_strndup (tmp, max_chars_per_title);
           g_free (tmp);
         }
       te->widget = NULL;
@@ -485,8 +471,11 @@ display_entry (MetaTabPopup *popup,
       gdk_region_destroy (region);
   
       /* This should piss off gtk a bit, but we don't want to raise
-       * above the tab popup
+       * above the tab popup.  So, instead of calling gtk_widget_show,
+       * we manually set the window as mapped and then manually map it
+       * with gdk functions.
        */
+      GTK_WIDGET_SET_FLAGS (popup->outline_window, GTK_MAPPED);
       gdk_window_show_unraised (popup->outline_window->window);
     }
 
@@ -819,6 +808,40 @@ meta_select_workspace_class_init (MetaSelectWorkspaceClass *klass)
   widget_class->expose_event = meta_select_workspace_expose_event;
 }
 
+/**
+ * meta_convert_meta_to_wnck() converts a MetaWindow to a
+ * WnckWindowDisplayInfo window that is used to build a thumbnail of a
+ * workspace.
+ **/
+static WnckWindowDisplayInfo
+meta_convert_meta_to_wnck (MetaWindow *window, MetaScreen *screen)
+{
+  WnckWindowDisplayInfo wnck_window;
+  wnck_window.icon = window->icon;
+  wnck_window.mini_icon = window->mini_icon;
+  
+  wnck_window.is_active = FALSE;
+  if (window == window->display->expected_focus_window)
+    wnck_window.is_active = TRUE;
+
+  if (window->frame)
+    {
+      wnck_window.x = window->frame->rect.x;
+      wnck_window.y = window->frame->rect.y;
+      wnck_window.width = window->frame->rect.width;
+      wnck_window.height = window->frame->rect.height;
+    }
+  else
+    {
+      wnck_window.x = window->rect.x;
+      wnck_window.y = window->rect.y;
+      wnck_window.width = window->rect.width;
+      wnck_window.height = window->rect.height;
+    }
+  return wnck_window;
+}
+
+
 static gboolean
 meta_select_workspace_expose_event (GtkWidget      *widget,
                                     GdkEventExpose *event)
@@ -847,7 +870,7 @@ meta_select_workspace_expose_event (GtkWidget      *widget,
                           workspace != workspace->screen->active_workspace;
 
       if (window->skip_pager || 
-          window->minimized || 
+          !meta_window_showing_on_its_workspace (window) ||
           window->unmaps_pending ||
           ignoreable_sticky)
         {
@@ -855,26 +878,7 @@ meta_select_workspace_expose_event (GtkWidget      *widget,
         }
       else
         {
-          windows[i].icon = window->icon;
-          windows[i].mini_icon = window->mini_icon;
-          windows[i].is_active = 
-            (window == window->display->expected_focus_window);
-
-          if (window->frame)
-            {
-              windows[i].x = window->frame->rect.x;
-              windows[i].y = window->frame->rect.y;
-              windows[i].width = window->frame->rect.width;
-              windows[i].height = window->frame->rect.height;
-            }
-          else
-            {                
-              windows[i].x = window->rect.x;
-              windows[i].y = window->rect.y;
-              windows[i].width = window->rect.width;
-              windows[i].height = window->rect.height;
-            }
-
+          windows[i] = meta_convert_meta_to_wnck (window, workspace->screen);
           i++;
         }
       tmp = tmp->next;
